@@ -10,7 +10,7 @@ from .core.ExprVisitor import ExprVisitor
 from .core.properties import PropertyRuleBook, BuiltinUserProperty
 from .core.namespace import NamespaceRegistry
 from .core.elaborate import ElabExpressionsListener, PrePlacementValidateListener, LateElabListener
-from .core.elaborate import StructuralPlacementListener
+from .core.elaborate import StructuralPlacementListener, LateElabRevisitor
 from .core.validate import ValidateListener
 from . import ast
 from . import component as comp
@@ -382,13 +382,17 @@ class RDLCompiler:
         )
 
         # Resolve address and field placement
+        late_elab_listener = LateElabListener(self.msg, self.env)
         walker.RDLWalker(skip_not_present=False).walk(
             root_node,
             PrePlacementValidateListener(self.msg),
             StructuralPlacementListener(self.msg),
-            LateElabListener(self.msg, self.env)
+            late_elab_listener
         )
 
+        # re-visit nodes a 2nd time as-needed to complete elaboration
+        LateElabRevisitor(late_elab_listener.node_needs_revisit)
+
         # Validate design
         # Only need to validate nodes that are present
         walker.RDLWalker(skip_not_present=True).walk(root_node, ValidateListener(self.env))
@@ -420,7 +424,7 @@ class RDLCompiler:
 
         .. versionadded:: 1.8
         """
-        # Create local message handler that suppresses the usual ouput
+        # Create local message handler that suppresses the usual output
         # to stderr.
         # Instead raises ValueError on any error
         msg_printer = messages.MessageExceptionRaiser()
