@@ -58,7 +58,7 @@ class ComponentVisitor(BaseVisitor):
         self._tmp_comp_def = None # type: comp.Component
         self._tmp_comp_inst = None # type: comp.Component
         self._tmp_inst_type = None # type: CommonToken
-        self._tmp_alias_primary_inst = None # type: Optional[comp.Component]
+        self._tmp_alias_primary_inst = None # type: Optional[comp.Reg]
 
     #---------------------------------------------------------------------------
     # Component Definitions
@@ -317,9 +317,14 @@ class ComponentVisitor(BaseVisitor):
                     src_ref_from_antlr(inst_type)
                 )
         elif isinstance(comp_inst, comp.Reg):
-            comp_inst.external = False
-            if (inst_type is not None) and (inst_type.type == SystemRDLParser.EXTERNAL_kw):
-                comp_inst.external = True
+            if inst_type is not None:
+                if inst_type.type == SystemRDLParser.EXTERNAL_kw:
+                    comp_inst.external = True
+                elif inst_type.type == SystemRDLParser.INTERNAL_kw:
+                    comp_inst.external = False
+            else:
+                # Leave as None. Elaborate step will resolve later.
+                comp_inst.external = None
 
         elif isinstance(comp_inst, comp.Regfile):
             if inst_type is not None:
@@ -904,6 +909,15 @@ class RootVisitor(ComponentVisitor):
         self.visitChildren(ctx)
         self.apply_dynamic_properties()
 
+    def visitLocal_property_assignment(self, ctx: SystemRDLParser.Local_property_assignmentContext) -> None:
+        # The only local assignments allowed in Root are default assignments
+        if ctx.DEFAULT_kw() is None:
+            self.msg.error(
+                "Illegal property assignment in root namespace",
+                src_ref_from_antlr(ctx)
+            )
+        super().visitLocal_property_assignment(ctx)
+
 
     def define_component(self, body: SystemRDLParser.Component_bodyContext, type_token: 'CommonToken', def_name: str, param_defs: List[Parameter]) -> comp.Component:
         comp_def = super().define_component(body, type_token, def_name, param_defs)
