@@ -235,21 +235,21 @@ class PropertyRule:
         if isinstance(value, int):
             if node.width < value.bit_length():
                 self.env.msg.error(
-                    "A counter's %s cannot refrence a value wider than the counter itself."
+                    "A counter's %s cannot reference a value wider than the counter itself."
                     % (self.get_name()),
                     node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
                 )
         elif isinstance(value, m_node.VectorNode):
             if node.width < value.width:
                 self.env.msg.error(
-                    "A counter's %s cannot refrence a value wider than the counter itself."
+                    "A counter's %s cannot reference a value wider than the counter itself."
                     % (self.get_name()),
                     node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
                 )
         elif isinstance(value, rdltypes.PropertyReference) and value.width is not None:
             if node.width < value.width:
                 self.env.msg.error(
-                    "A counter's %s cannot refrence a value wider than the counter itself."
+                    "A counter's %s cannot reference a value wider than the counter itself."
                     % (self.get_name()),
                     node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
                 )
@@ -265,6 +265,7 @@ class PropertyRuleBoolPair(PropertyRule):
         If not explicitly set, check if the opposite was set first before returning
         default
         """
+        # Check for explicit assignment to avoid infinite loop via get_default()
         if self.opposite_property in node.inst.properties:
             return not node.inst.properties[self.opposite_property]
         else:
@@ -327,7 +328,7 @@ class Prop_dontcompare(PropertyRule):
             comp_def.properties[self.get_name()] = value
 
     def validate(self, node: m_node.Node, value: Any) -> None:
-        donttest = node.get_property("donttest")
+        donttest = node.get_property('donttest')
 
         if isinstance(node, m_node.FieldNode):
             # 5.2.2.1-a: If value is a bit mask, the mask shall have the same width
@@ -560,7 +561,7 @@ class Prop_cpuif_reset(PropertyRule):
 
     def validate(self, node: m_node.Node, value: Any) -> None:
         if value is True:
-            if not node.get_property("activehigh") and not node.get_property("activelow"):
+            if not node.get_property('activehigh') and not node.get_property('activelow'):
                 self.env.msg.error(
                     "Signal '%s' sets the 'cpuif_reset' property but does not specify whether it is activehigh/activelow"
                     % (node.inst_name),
@@ -583,7 +584,7 @@ class Prop_field_reset(PropertyRule):
 
     def validate(self, node: m_node.Node, value: Any) -> None:
         if value is True:
-            if not node.get_property("activehigh") and not node.get_property("activelow"):
+            if not node.get_property('activehigh') and not node.get_property('activelow'):
                 self.env.msg.error(
                     "Signal '%s' sets the 'field_reset' property but does not specify whether it is activehigh/activelow"
                     % (node.inst_name),
@@ -677,6 +678,14 @@ class Prop_next(PropertyRule):
         self._validate_ref_width(node, value)
         self._validate_ref_is_present(node, value)
 
+        # Field shall be hardware writable
+        # Example in 17.2.8 makes a passing comment that suggests this is a requirement.
+        if not node.is_hw_writable:
+            self.env.msg.error(
+                "Use of the 'next' property requires the field to be hardware-writable.",
+                node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
+            )
+
 
 class Prop_reset(PropertyRule):
     """
@@ -735,7 +744,7 @@ class Prop_resetsignal(PropertyRule):
         self._validate_ref_width_is_1(node, value)
         self._validate_ref_is_present(node, value)
 
-        if not value.get_property("activehigh") and not value.get_property("activelow"):
+        if not value.get_property('activehigh') and not value.get_property('activelow'):
             self.env.msg.error(
                 "Signal '%s' referenced in 'resetsignal' does not specify whether it is activehigh/activelow"
                 % (value.inst_name),
@@ -775,6 +784,7 @@ class Prop_rclr(PropertyRule):
         """
         If not explicitly set, check if onread sets the equivalent
         """
+        # Check for explicit assignment to avoid infinite loop via get_default()
         if node.inst.properties.get("onread", None) == rdltypes.OnReadType.rclr:
             return True
         else:
@@ -805,6 +815,7 @@ class Prop_rset(PropertyRule):
         """
         If not explicitly set, check if onread sets the equivalent
         """
+        # Check for explicit assignment to avoid infinite loop via get_default()
         if node.inst.properties.get("onread", None) == rdltypes.OnReadType.rset:
             return True
         else:
@@ -835,6 +846,7 @@ class Prop_onread(PropertyRule):
         """
         If not explicitly set, check if rset or rclr imply the value
         """
+        # Check for explicit assignment to avoid infinite loop via get_default()
         if node.inst.properties.get("rset", False):
             return rdltypes.OnReadType.rset
         elif node.inst.properties.get("rclr", False):
@@ -876,6 +888,7 @@ class Prop_woclr(PropertyRule):
         """
         If not explicitly set, check if onwrite sets the equivalent
         """
+        # Check for explicit assignment to avoid infinite loop via get_default()
         if node.inst.properties.get("onwrite", None) == rdltypes.OnWriteType.woclr:
             return True
         else:
@@ -904,8 +917,9 @@ class Prop_woset(PropertyRule):
 
     def get_default(self, node: m_node.Node) -> bool:
         """
-        If not explicitly set, check if onread sets the equivalent
+        If not explicitly set, check if onwrite sets the equivalent
         """
+        # Check for explicit assignment to avoid infinite loop via get_default()
         if node.inst.properties.get("onwrite", None) == rdltypes.OnWriteType.woset:
             return True
         else:
@@ -936,6 +950,7 @@ class Prop_onwrite(PropertyRule):
         """
         If not explicitly set, check if woset or woclr imply the value
         """
+        # Check for explicit assignment to avoid infinite loop via get_default()
         if node.inst.properties.get("woset", False):
             return rdltypes.OnWriteType.woset
         elif node.inst.properties.get("woclr", False):
@@ -1116,7 +1131,7 @@ class Prop_singlepulse(PropertyRule):
 
 class Prop_we(PropertyRule):
     bindable_to = {comp.Field}
-    valid_types = (bool, comp.Signal, comp.Field,  rdltypes.PropertyReference,)
+    valid_types = (bool, comp.Signal, comp.Field, rdltypes.PropertyReference,)
     default = False
     dyn_assign_allowed = True
     mutex_group = "C"
@@ -1389,6 +1404,16 @@ class Prop_overflow(CounterProperty):
     dyn_assign_allowed = True
     mutex_group = None
 
+    def validate(self, node: m_node.Node, value: Any) -> None:
+        super().validate(node, value)
+
+        if node.get_property('incrsaturate') is not False:
+            self.env.msg.error(
+                "Use of 'overflow' property is meaningless. Counter sets the "
+                "'incrsaturate' property which makes it unable to overflow",
+                node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
+            )
+
 
 class Prop_underflow(CounterProperty):
     bindable_to = {comp.Field}
@@ -1397,6 +1422,16 @@ class Prop_underflow(CounterProperty):
     dyn_assign_allowed = True
     mutex_group = None
 
+    def validate(self, node: m_node.Node, value: Any) -> None:
+        super().validate(node, value)
+
+        if node.get_property('decrsaturate') is not False:
+            self.env.msg.error(
+                "Use of 'underflow' property is meaningless. Counter sets the "
+                "'decrsaturate' property which makes it unable to underflow",
+                node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
+            )
+
 
 class Prop_incr(CounterProperty):
     bindable_to = {comp.Field}
@@ -1441,6 +1476,17 @@ class Prop_incrwidth(CounterProperty):
     dyn_assign_allowed = True
     mutex_group = "F"
 
+    def validate(self, node: m_node.Node, value: Any) -> None:
+        super().validate(node, value)
+        assert isinstance(node, m_node.FieldNode)
+
+        if not 1 <= value <= node.width:
+            self.env.msg.error(
+                "A counter's 'incrwidth' must be between 1 and the counter's width (%d)"
+                % node.width,
+                node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
+            )
+
 
 class Prop_decrvalue(CounterProperty):
     bindable_to = {comp.Field}
@@ -1485,6 +1531,17 @@ class Prop_decrwidth(CounterProperty):
     dyn_assign_allowed = True
     mutex_group = "G"
 
+    def validate(self, node: m_node.Node, value: Any) -> None:
+        super().validate(node, value)
+        assert isinstance(node, m_node.FieldNode)
+
+        if not 1 <= value <= node.width:
+            self.env.msg.error(
+                "A counter's 'decrwidth' must be between 1 and the counter's width (%d)"
+                % node.width,
+                node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
+            )
+
 
 class Prop_decrsaturate(CounterProperty):
     bindable_to = {comp.Field}
@@ -1528,17 +1585,23 @@ class Prop_intr(PropertyRule):
 class Prop_intr_type(PropertyRule):
     bindable_to = {comp.Field}
     valid_types = (rdltypes.InterruptType,)
-    default = rdltypes.InterruptType.level
+    default = None
     dyn_assign_allowed = True
     mutex_group = None
 
     @classmethod
     def get_name_cls(cls) -> str:
         # Interrupt modifier type is a "special" hidden property
-        # Intentinally override the property name to something that is impossible
+        # Intentionally override the property name to something that is impossible
         # to define in RDL and collide with: contains a space!
         return "intr type"
 
+    def get_default(self, node: m_node.Node) -> Any:
+        if node.get_property('intr'):
+            # If unspecified, interrupt fields are level-sensitive by default
+            return rdltypes.InterruptType.level
+        return None
+
 
 class Prop_enable(PropertyRule):
     bindable_to = {comp.Field}
@@ -1552,6 +1615,13 @@ class Prop_enable(PropertyRule):
         self._validate_ref_width(node, value)
         self._validate_ref_is_present(node, value)
 
+        if value:
+            if not node.get_property('intr'):
+                self.env.msg.error(
+                    "The 'enable' property can only be used on interrupt fields.",
+                    node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
+                )
+
 
 class Prop_mask(PropertyRule):
     bindable_to = {comp.Field}
@@ -1565,6 +1635,13 @@ class Prop_mask(PropertyRule):
         self._validate_ref_width(node, value)
         self._validate_ref_is_present(node, value)
 
+        if value:
+            if not node.get_property('intr'):
+                self.env.msg.error(
+                    "The 'mask' property can only be used on interrupt fields.",
+                    node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
+                )
+
 
 class Prop_haltenable(PropertyRule):
     bindable_to = {comp.Field}
@@ -1578,6 +1655,13 @@ class Prop_haltenable(PropertyRule):
         self._validate_ref_width(node, value)
         self._validate_ref_is_present(node, value)
 
+        if value:
+            if not node.get_property('intr'):
+                self.env.msg.error(
+                    "The 'haltenable' property can only be used on interrupt fields.",
+                    node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
+                )
+
 
 class Prop_haltmask(PropertyRule):
     bindable_to = {comp.Field}
@@ -1591,6 +1675,13 @@ class Prop_haltmask(PropertyRule):
         self._validate_ref_width(node, value)
         self._validate_ref_is_present(node, value)
 
+        if value:
+            if not node.get_property('intr'):
+                self.env.msg.error(
+                    "The 'haltmask' property can only be used on interrupt fields.",
+                    node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
+                )
+
 
 class Prop_sticky(PropertyRule):
     bindable_to = {comp.Field}
@@ -1599,6 +1690,50 @@ class Prop_sticky(PropertyRule):
     dyn_assign_allowed = True
     mutex_group = "I"
 
+    def validate(self, node: m_node.Node, value: Any) -> None:
+        assert isinstance(node, m_node.FieldNode)
+        if value is True:
+            # 'sticky' property doesnt quite make sense for edge-senstive interrupts
+            intr_type = node.get_property('intr type')
+            if intr_type in {
+                rdltypes.InterruptType.posedge,
+                rdltypes.InterruptType.negedge,
+                rdltypes.InterruptType.bothedge,
+            }:
+                self.env.msg.error(
+                    "Whole-field stickiness only makes sense in level-senstive interrupts, "
+                    "but this field is defined as '%s intr'. "
+                    "Did you mean to use the 'stickybit' property instead of 'sticky'?"
+                    % intr_type.name,
+                    node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
+                )
+
+            # Use of we/wel qualifier conflicts with sticky property
+            if node.get_property('we'):
+                self.env.msg.error(
+                    "Use of a hardware write-enable on field '%s' does not make "
+                    "sense because it is defined as 'sticky'. Sticky fields already "
+                    "implicitly control their hardware write-enable behavior based on the input value."
+                    % (node.inst_name),
+                    node.inst.property_src_ref.get('we', node.inst.inst_src_ref)
+                )
+            if node.get_property('wel'):
+                self.env.msg.error(
+                    "Use of a hardware write-enable on field '%s' does not make "
+                    "sense because it is defined as 'sticky'. Sticky fields already "
+                    "implicitly control their hardware write-enable behavior based on the input value."
+                    % (node.inst_name),
+                    node.inst.property_src_ref.get('wel', node.inst.inst_src_ref)
+                )
+
+            if not node.is_hw_writable:
+                self.env.msg.error(
+                    "Sticky fields shall be hardware-writable",
+                    node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
+                )
+
+
+
 
 class Prop_stickybit(PropertyRule):
     bindable_to = {comp.Field}
@@ -1611,14 +1746,44 @@ class Prop_stickybit(PropertyRule):
         """
         Unless specified otherwise, intr fields are implicitly stickybit
         """
-        if node.inst.properties.get("intr", False):
-            # Interrupt is set!
-            # Default is implicitly stickybit, unless the mutually-exclusive
-            # sticky property was set instead
-            return not node.inst.properties.get("sticky", False)
+        if node.get_property('intr'):
+            # Field is an interrupt
+            if node.get_property('sticky'):
+                # ... and it was defined as multibit sticky. Not stickybit
+                return False
+            else:
+                # By default, implies stickybit
+                return True
         else:
             return False
 
+    def validate(self, node: m_node.Node, value: Any) -> None:
+        assert isinstance(node, m_node.FieldNode)
+        if value is True:
+            # Use of we/wel qualifier conflicts with stickybit property
+            if node.get_property('we'):
+                self.env.msg.error(
+                    "Use of a hardware write-enable on field '%s' does not make "
+                    "sense because it is defined as 'stickybit'. Stickybit fields already "
+                    "implicitly control their hardware write-enable behavior based on the input value."
+                    % (node.inst_name),
+                    node.inst.property_src_ref.get('we', node.inst.inst_src_ref)
+                )
+            if node.get_property('wel'):
+                self.env.msg.error(
+                    "Use of a hardware write-enable on field '%s' does not make "
+                    "sense because it is defined as 'stickybit'. Stickybit fields already "
+                    "implicitly control their hardware write-enable behavior based on the input value."
+                    % (node.inst_name),
+                    node.inst.property_src_ref.get('wel', node.inst.inst_src_ref)
+                )
+
+            if not node.is_hw_writable:
+                self.env.msg.error(
+                    "Stickybit fields shall be hardware-writable",
+                    node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
+                )
+
 
 #-------------------------------------------------------------------------------
 # Misc properties
@@ -1991,7 +2156,7 @@ class PropRef_xored(rdltypes.PropertyReference):
 #-------------------------------------------------------------------------------
 class CounterPropRef(rdltypes.PropertyReference):
     def _validate(self) -> None:
-        if not self.node.get_property("counter"):
+        if not self.node.get_property('counter'):
             self.env.msg.error(
                 "Reference to property '%s' is illegal because '%s' is not a counter"
                 % (self.name, self.node.inst_name),
@@ -2112,6 +2277,17 @@ class PropRef_overflow(CounterPropRef):
     def width(self) -> Optional[int]:
         return 1
 
+    def _validate(self) -> None:
+        super()._validate()
+
+        if self.node.get_property('incrsaturate') is not False:
+            self.env.msg.error(
+                "Reference to property '%s' is illegal because the target field will never overflow"
+                % self.name,
+                self.src_ref
+            )
+
+
 class PropRef_underflow(CounterPropRef):
     """
     asserted when counter underflows or wraps.
@@ -2122,6 +2298,16 @@ class PropRef_underflow(CounterPropRef):
     def width(self) -> Optional[int]:
         return 1
 
+    def _validate(self) -> None:
+        super()._validate()
+
+        if self.node.get_property('decrsaturate') is not False:
+            self.env.msg.error(
+                "Reference to property '%s' is illegal because the target field will never underflow"
+                % self.name,
+                self.src_ref
+            )
+
 #-------------------------------------------------------------------------------
 # Access
 #-------------------------------------------------------------------------------
@@ -2200,7 +2386,7 @@ class PropRef_intr(rdltypes.PropertyReference):
     def _validate(self) -> None:
         # validate reg contains at least one field that is intr
         for field in self.node.fields():
-            if field.get_property("intr"):
+            if field.get_property('intr'):
                 break
         else:
             self.env.msg.error(
@@ -2222,7 +2408,7 @@ class PropRef_halt(rdltypes.PropertyReference):
         # 10.8.1-c: shall only be present if haltmask or haltenable is
         # specified on at least one field in the register.
         for field in self.node.fields():
-            if field.get_property("haltenable") or field.get_property("haltmask"):
+            if field.get_property('haltenable') or field.get_property('haltmask'):
                 break
         else:
             self.env.msg.error(
