@@ -49,8 +49,8 @@ class RDLImporter:
 
     # ------------------------ create_xxx_definition() -------------------------
     def _create_definition(self, cls: "Type[ComponentClass]", type_name: Optional[str] = None, src_ref: Optional[SourceRefBase] = None) -> "ComponentClass":
-        if (type_name is not None) and not re.fullmatch(r"\w+", type_name):
-            raise ValueError("Type name has invalid characters")
+        if (type_name is not None) and not re.fullmatch(r"[a-zA-Z]\w*", type_name):
+            raise ValueError("Type name has invalid characters: '%s'" % type_name)
         C = cls()
         C.type_name = type_name
         C.def_src_ref = src_ref or self.default_src_ref
@@ -133,8 +133,8 @@ class RDLImporter:
     def _instantiate(self, comp_def: "ComponentClass", inst_name: str, src_ref: Optional[SourceRefBase] = None) -> "ComponentClass":
         if comp_def.is_instance:
             raise ValueError("Component is already instantiated")
-        if not re.fullmatch(r"\w+", inst_name):
-            raise ValueError("Instance name has invalid characters")
+        if not re.fullmatch(r"[a-zA-Z]\w*", inst_name):
+            raise ValueError("Instance name has invalid characters: '%s'" % inst_name)
 
         if comp_def.type_name is None:
             # anonymous definition. No need to copy since it can only be used once
