@@ -1,6 +1,6 @@
 import re
 import itertools
-from copy import deepcopy
+from copy import deepcopy, copy
 from collections import deque
 from typing import TYPE_CHECKING, Optional, Iterator, Any, List, Dict, Union
 
@@ -613,7 +613,7 @@ class Node:
         .. versionchanged:: 1.6
             Added ``markdown_inst`` option.
         """
-        desc_str = self.get_property("desc")
+        desc_str = self.get_property('desc')
         if desc_str is None:
             return None
         return rdlformatcode.rdlfc_to_html(desc_str, self, md=markdown_inst)
@@ -634,7 +634,7 @@ class Node:
 
         .. versionadded:: 1.8
         """
-        name_str = self.get_property("name", default=None)
+        name_str = self.get_property('name', default=None)
         if name_str is None:
             return None
         return rdlformatcode.rdlfc_to_html(name_str, self, is_desc=False)
@@ -1017,7 +1017,6 @@ class FieldNode(VectorNode):
             (hw in (rdltypes.AccessType.rw, rdltypes.AccessType.rw1,
                     rdltypes.AccessType.w, rdltypes.AccessType.w1))
             or self.get_property('counter')
-            or (self.get_property('next') is not None)
             or self.get_property('hwset')
             or self.get_property('hwclr')
         )
@@ -1067,11 +1066,34 @@ class FieldNode(VectorNode):
         """
         True if combination of field access properties imply that the field
         implements a storage element.
-        """
 
-        # 9.4.1, Table 12
+
+        .. versionchanged:: 1.22
+            Counter, interrupt, stickybit, and sticky fields always implement
+            storage, regardless of sw/hw access.
+
+        .. versionchanged:: 1.23
+            Alias fields never implement storage.
+            A primary field may inherit a storage element depending on the access
+            modes of aliases that augment access to it.
+        """
+        if self.is_alias:
+            # A field that is an alias never implements storage.
+            # Storage element, if any, resides in the alias's primary.
+            return False
+
         sw = self.get_property('sw')
         hw = self.get_property('hw')
+        onread = self.get_property('onread')
+
+        # Aliases of a primary may have additive access modes that imply
+        # that the primary actually implements storage.
+        # accumulate access modes
+        for alias_field in self.aliases():
+            sw += alias_field.get_property('sw')
+            onread = onread or alias_field.get_property('onread')
+
+        # 9.4.1, Table 12
         if sw in (rdltypes.AccessType.rw, rdltypes.AccessType.rw1):
             # Software can read and write, implying a storage element
             return True
@@ -1083,7 +1105,6 @@ class FieldNode(VectorNode):
             return True
 
 
-        onread = self.get_property('onread')
         if onread is not None:
             # 9.6.1-c: Onread side-effects imply storage regardless of whether
             # or not the field is writable by sw
@@ -1094,6 +1115,18 @@ class FieldNode(VectorNode):
             # Not in spec, but these imply that a storage element exists
             return True
 
+        if self.get_property('counter'):
+            # All counters implicitly implement storage
+            return True
+
+        if self.get_property('intr'):
+            # All interrupt fields implicitly implement storage
+            return True
+
+        if self.get_property('stickybit') or self.get_property('sticky'):
+            # All sticky fields implicitly implement storage
+            return True
+
         return False
 
     @property
@@ -1144,6 +1177,80 @@ class FieldNode(VectorNode):
 
         return False
 
+
+    @property
+    def is_alias(self) -> bool:
+        """
+        Indicates whether this field is an alias
+        (is contained inside an alias register)
+
+
+        .. versionadded:: 1.23
+        """
+        assert isinstance(self.parent, RegNode)
+        return self.parent.is_alias
+
+
+    @property
+    def alias_primary(self) -> 'FieldNode':
+        """
+        Returns the FieldNode that is associated with this alias's primary register.
+
+        Raises ValueError if this field is not an alias
+
+
+        .. versionadded:: 1.23
+        """
+        if not self.is_alias:
+            raise ValueError("Field is not an alias")
+
+        assert isinstance(self.parent, RegNode)
+        primary_reg = self.parent.alias_primary
+        primary_field = primary_reg.get_child_by_name(self.inst_name)
+        assert isinstance(primary_field, FieldNode)
+        return primary_field
+
+
+    @property
+    def has_aliases(self) -> bool:
+        """
+        Returns True if this field has aliases
+
+
+        .. versionadded:: 1.23
+        """
+        for _ in self.aliases():
+            return True
+        return False
+
+
+    def aliases(self, skip_not_present: bool = True) -> Iterator['FieldNode']:
+        """
+        Returns an iterator of all the fields that are aliases of this primary field
+
+        Parameters
+        ----------
+        skip_not_present : bool
+            If True, skips aliases whose 'ispresent' property is set to False
+
+
+        .. versionadded:: 1.23
+        """
+        assert isinstance(self.parent, RegNode)
+        for alias_reg in self.parent.aliases(skip_not_present=skip_not_present):
+            alias_field = alias_reg.get_child_by_name(self.inst_name)
+
+            if not isinstance(alias_field, FieldNode):
+                # not found in this alias
+                continue
+
+            if skip_not_present and not alias_field.get_property('ispresent'):
+                continue
+
+            yield alias_field
+
+
+
 #===============================================================================
 class RegNode(AddressableNode):
 
@@ -1201,6 +1308,121 @@ class RegNode(AddressableNode):
                 return True
         return False
 
+    @property
+    def is_interrupt_reg(self) -> bool:
+        """
+        Register contains one or more interrupt fields and therefore produces an
+        interrupt output signal.
+
+
+        .. versionadded:: 1.22
+        """
+        for field in self.fields():
+            if field.get_property('intr'):
+                return True
+        return False
+
+    @property
+    def is_halt_reg(self) -> bool:
+        """
+        Register contains one or more interrupt fields that use ``haltenable``
+        or ``haltmask`` and therefore produces a halt output signal.
+
+
+        .. versionadded:: 1.22
+        """
+        for field in self.fields():
+            if field.get_property('haltenable') or field.get_property('haltmask'):
+                return True
+        return False
+
+
+    @property
+    def is_alias(self) -> bool:
+        """
+        Indicates whether this register is an alias.
+
+
+        .. versionadded:: 1.23
+        """
+        assert isinstance(self.inst, comp.Reg)
+        return self.inst.is_alias
+
+
+    @property
+    def alias_primary(self) -> 'RegNode':
+        """
+        Returns the RegNode of this alias's primary register.
+
+        Raises ValueError if this register is not an alias
+
+
+        .. versionadded:: 1.23
+        """
+        if not self.is_alias:
+            raise ValueError("Register is not an alias")
+
+        # Limitations in grammar make it so that an alias primary can only be
+        # referenced by a single ID token. This means the primary cannot be a
+        # hierarchical path, or anything else fancy. This implicitly guarantees
+        # that the alias primary is a sibling in the hierarchy.
+        # Simply look up the sibling by-name.
+        assert isinstance(self.inst, comp.Reg)
+        name = self.inst.alias_primary_inst.inst_name
+        primary_reg = self.parent.get_child_by_name(name)
+        assert isinstance(primary_reg, RegNode)
+
+        if self.is_array and primary_reg.is_array:
+            # Both are arrays.
+            # Mirror the current index onto the primary that is returned
+            primary_reg.current_idx = copy(self.current_idx)
+
+        return primary_reg
+
+
+    @property
+    def has_aliases(self) -> bool:
+        """
+        Returns True if this register has aliases
+
+
+        .. versionadded:: 1.23
+        """
+        assert isinstance(self.inst, comp.Reg)
+        return bool(self.inst._alias_names)
+
+
+    def aliases(self, skip_not_present: bool = True) -> Iterator['RegNode']:
+        """
+        Returns an iterator of all the registers that are aliases of this primary register
+
+        Parameters
+        ----------
+        skip_not_present : bool
+            If True, skips aliases whose 'ispresent' property is set to False
+
+
+        .. versionadded:: 1.23
+        """
+        assert isinstance(self.inst, comp.Reg)
+
+        for reg_name in self.inst._alias_names:
+            alias_reg = self.parent.get_child_by_name(reg_name)
+            assert isinstance(alias_reg, RegNode)
+
+            if skip_not_present and not alias_reg.get_property('ispresent'):
+                continue
+
+            if self.is_array and alias_reg.is_array:
+                # Both are arrays.
+                # Mirror the current index onto the alias that is returned
+                alias_reg.current_idx = copy(self.current_idx)
+
+            yield alias_reg
+
+
+
+
 #===============================================================================
 class RegfileNode(AddressableNode):
 
