@@ -39,14 +39,19 @@ else:
     PreElabRDLType = Any # type: ignore
 
 #===============================================================================
-class BuiltinEnum(enum.Enum):
-    # backport equivalent py3.6 support for auto enumeration
-    def __new__(cls) -> 'BuiltinEnum':
+class AutoNoValueEnum(enum.Enum):
+    def __new__(cls) -> 'AutoNoValueEnum':
         value = len(cls.__members__) + 1
         obj = object.__new__(cls)
         obj._value_ = value
         return obj
 
+    def __repr__(self) -> str:
+        return '<%s.%s>' % (self.__class__.__name__, self.name)
+
+class BuiltinEnum(AutoNoValueEnum):
+    pass
+
 #===============================================================================
 class AccessType(BuiltinEnum):
     #: Not Accessible
@@ -67,6 +72,51 @@ class AccessType(BuiltinEnum):
     #: Write-only. After a reset occurs, can only be written once.
     w1 = ()
 
+    def __add__(self, other: Any) -> 'AccessType':
+        """
+        Add operator combines AccessTypes into the more permissive superset of
+        the two addends.
+
+        For example: ``r + w --> rw``
+
+
+        .. versionadded:: 1.23
+        """
+        if not isinstance(other, AccessType):
+            raise TypeError("unsupported addend types: %s + %s" % (repr(self), repr(other)))
+
+        pair = {other, self}
+
+        # addends are the same. No change
+        if len(pair) == 1:
+            return self
+
+        # rw superscedes all others
+        if AccessType.rw in pair:
+            return AccessType.rw
+
+        # na is supersceded by all others
+        if self == AccessType.na:
+            return other
+        if other == AccessType.na:
+            return self
+
+        if pair == {AccessType.r, AccessType.w}:
+            return AccessType.rw
+        if pair == {AccessType.r, AccessType.rw1}:
+            return AccessType.rw1
+        if pair == {AccessType.r, AccessType.w1}:
+            return AccessType.rw1
+        if pair == {AccessType.w, AccessType.rw1}:
+            return AccessType.rw
+        if pair == {AccessType.w, AccessType.w1}:
+            return AccessType.w
+        if pair == {AccessType.rw1, AccessType.w1}:
+            return AccessType.rw1
+
+        # unreachable
+        raise RuntimeError
+
 class OnReadType(BuiltinEnum):
     #: Cleared on read
     rclr = ()
@@ -132,14 +182,21 @@ class InterruptType(BuiltinEnum):
             negedge intr;
         };
 
-    The modifier is stored in the internal "intr type" property. (note the intentional space in the name)
+    The modifier is stored in the internal "intr type" property. (note the
+    intentional space in the name)
 
     It can be fetched the same way as other properties:
 
     .. code-block:: python
 
-        intr_type = my_field_node.get_property("intr type")
+        intr_type = my_field_node.get_property('intr type')
 
+
+    .. note::
+
+        The ``nonsticky`` interrupt type is intentionally omitted from this
+        enumeration since it is not really a distinct interrupt type. Its use in
+        SystemRDL implies an assignment of ``stickybit = false``.
     """
     #: Interrupt when asserted and maintained
     level = ()
@@ -322,7 +379,7 @@ class UserStruct:
 
     .. code-block:: python
 
-        prop = node.get_property("my_struct_prop")
+        prop = node.get_property('my_struct_prop')
 
         foo = prop.foo
         bar = getattr(prop, "bar")
@@ -535,7 +592,7 @@ class PropertyReference:
         fieldX = my_reg.get_child_by_name("fieldX")
         fieldY = my_reg.get_child_by_name("fieldY")
 
-        next_prop = fieldX.get_property("next")
+        next_prop = fieldX.get_property('next')
         print(next_prop.node == fieldY) # prints: True
         print(next_prop.name) # prints: "intr"
 
